@@ -114,6 +114,26 @@ bodyaccess_log_cb(struct req *req, void *priv, void *ptr, size_t len)
 
 	return (bodyaccess_log(priv, ptr, len));
 }
+#elif defined(HAVE_OBJITERATE_F) && defined(OBJ_ITER_FLUSH)
+static int
+bodyaccess_bcat_cb(void *priv, unsigned flush, const void *ptr, ssize_t len)
+{
+
+	AN(priv);
+
+	(void)flush;
+	return (VSB_bcat(priv, ptr, len));
+}
+
+static int
+bodyaccess_log_cb(void *priv, unsigned flush, const void *ptr, ssize_t len)
+{
+
+	AN(priv);
+
+	(void)flush;
+	return (bodyaccess_log(priv, ptr, len));
+}
 #elif defined(HAVE_OBJITERATE_F)
 static int
 bodyaccess_bcat_cb(void *priv, int flush, const void *ptr, ssize_t len)
@@ -146,7 +166,8 @@ bodyaccess_bcat(VRT_CTX, struct vsb *vsb)
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 
-	l = VRB_Iterate(ctx->req, bodyaccess_bcat_cb, vsb);
+	l = VRB_Iterate(ctx->req->wrk, ctx->vsl, ctx->req, bodyaccess_bcat_cb,
+	    vsb);
 	AZ(VSB_finish(vsb));
 	if (l < 0)
 		VSLb(ctx->vsl, SLT_VCL_Error,
@@ -161,7 +182,7 @@ vmod_hash_req_body(VRT_CTX)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 
-	if (ctx->req->req_body_status != REQ_BODY_CACHED) {
+	if (ctx->req->req_body_status != BS_CACHED) {
 		VSLb(ctx->vsl, SLT_VCL_Error,
 		   "Unbuffered req.body");
 		return;
@@ -187,10 +208,12 @@ vmod_hash_req_body(VRT_CTX)
 VCL_INT
 vmod_len_req_body(VRT_CTX)
 {
+	uint64_t u;
+
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 
-	if (ctx->req->req_body_status != REQ_BODY_CACHED) {
+	if (ctx->req->req_body_status != BS_CACHED) {
 		VSLb(ctx->vsl, SLT_VCL_Error,
 		   "Unbuffered req.body");
 		return (-1);
@@ -202,7 +225,10 @@ vmod_len_req_body(VRT_CTX)
 		return (-1);
 	}
 
-	return (ctx->req->req_bodybytes);
+	AZ(ObjGetU64(ctx->req->wrk, ctx->req->body_oc, OA_LEN, &u));
+	AZ(u > INT64_MAX);
+
+	return ((VCL_INT)u);
 }
 
 VCL_INT
@@ -216,7 +242,7 @@ vmod_rematch_req_body(VRT_CTX, struct vmod_priv *priv_call, VCL_STRING re)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 
-	if (ctx->req->req_body_status != REQ_BODY_CACHED) {
+	if (ctx->req->req_body_status != BS_CACHED) {
 		VSLb(ctx->vsl, SLT_VCL_Error,
 		   "Unbuffered req.body");
 		return(-1);
@@ -279,12 +305,13 @@ vmod_log_req_body(VRT_CTX, VCL_STRING prefix, VCL_INT length)
 	log_ctx.pfx = prefix;
 	log_ctx.len = length;
 
-	if (ctx->req->req_body_status != REQ_BODY_CACHED) {
+	if (ctx->req->req_body_status != BS_CACHED) {
 		VSLb(ctx->vsl, SLT_VCL_Error, "Unbuffered req.body");
 		return;
 	}
 
-	ret = VRB_Iterate(ctx->req, bodyaccess_log_cb, &log_ctx);
+	ret = VRB_Iterate(ctx->req->wrk, ctx->vsl, ctx->req, bodyaccess_log_cb,
+	    &log_ctx);
 
 	if (ret < 0) {
 		VSLb(ctx->vsl, SLT_VCL_Error,
