@@ -35,14 +35,16 @@
 
 #include "vmod_config.h"
 
-#include "cache/cache_director.h"
-#include "cache/cache_backend.h"
-
 #include "vsb.h"
 #include "vtim.h"
 
 #include "vcc_saintmode_if.h"
 
+static VCL_BOOL v_matchproto_(vdi_healthy_f)
+healthy(VRT_CTX, VCL_BACKEND, VCL_TIME *);
+static VCL_BACKEND v_matchproto_(vdi_resolve_f)
+resolve(VRT_CTX, VCL_BACKEND);
+
 struct trouble {
 	unsigned		magic;
 #define TROUBLE_MAGIC		0x4211ab21
@@ -54,7 +56,7 @@ struct trouble {
 struct vmod_saintmode_saintmode {
 	unsigned				magic;
 #define VMOD_SAINTMODE_MAGIC			0xa03756e4
-	struct director				sdir[1];
+	const struct director			*sdir;
 	const struct director			*be;
 	pthread_mutex_t				mtx;
 	unsigned				threshold;
@@ -63,6 +65,14 @@ struct vmod_saintmode_saintmode {
 	VTAILQ_HEAD(, trouble)			troublelist;
 };
 
+static const struct vdi_methods vmod_saintmode_methods[1] = {{
+	.magic =		VDI_METHODS_MAGIC,
+	.type =			"saintmode",
+	.healthy =		healthy,
+	.resolve =		resolve
+}};
+
+
 struct saintmode_objs {
 	unsigned				magic;
 #define SAINTMODE_OBJS_MAGIC			0x9aa7beec
@@ -253,23 +263,26 @@ is_digest_healthy(const struct director *dir,
 }
 
 /* All adapted from PHK's saintmode implementation in Varnish 3.0 */
-static unsigned
-healthy(const struct director *dir, const struct busyobj *bo, double *changed)
+static VCL_BOOL v_matchproto_(vdi_healthy_f)
+healthy(VRT_CTX, VCL_BACKEND dir, VCL_TIME *changed)
 {
 	struct vmod_saintmode_saintmode *sm;
+	const struct busyobj *bo;
 	unsigned retval;
 	const uint8_t* digest;
 	double t_prev;
 	struct vsl_log* vsl;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
 	CAST_OBJ_NOTNULL(sm, dir->priv, VMOD_SAINTMODE_MAGIC);
 	CHECK_OBJ_NOTNULL(sm->be, DIRECTOR_MAGIC);
+	bo = ctx->bo;
 	CHECK_OBJ_ORNULL(bo, BUSYOBJ_MAGIC);
 
 	/* Saintmode is disabled, or list is empty */
 	if (sm->threshold == 0 || sm->n_trouble == 0)
-		return (sm->be->healthy(sm->be, bo, changed));
+		return (VRT_Healthy(ctx, sm->be, changed));
 
 	if (!bo) {
 		digest = NULL;
@@ -282,7 +295,7 @@ healthy(const struct director *dir, const struct busyobj *bo, double *changed)
 	}
 
 	retval = is_digest_healthy(dir, digest, t_prev, vsl);
-	return (retval ? sm->be->healthy(sm->be, bo, changed) : 0);
+	return (retval ? VRT_Healthy(ctx, sm->be, changed) : 0);
 }
 
 VCL_BOOL
@@ -303,19 +316,17 @@ vmod_saintmode_is_healthy(VRT_CTX, struct vmod_saintmode_saintmode *sm)
 		return  is_digest_healthy(sm->sdir, digest,
 					  ctx->req->t_prev, ctx->req->vsl);
 	} else
-		return healthy(sm->sdir, ctx->bo, NULL);
+		return healthy(ctx, sm->sdir, NULL);
 }
 
-static const struct director *
-resolve(const struct director *dir, struct worker *wrk, struct busyobj *bo) {
+static VCL_BACKEND v_matchproto_(vdi_resolve_f)
+resolve(VRT_CTX, VCL_BACKEND dir) {
 	struct vmod_saintmode_saintmode *sm;
-	double changed = 0.0;
 
 	CHECK_OBJ_NOTNULL(dir, DIRECTOR_MAGIC);
 	CAST_OBJ_NOTNULL(sm, dir->priv, VMOD_SAINTMODE_MAGIC);
-	(void)wrk;
 
-	if (!healthy(dir, bo, &changed))
+	if (!healthy(ctx, dir, NULL))
 		return (NULL);
 
 	return (sm->be);
@@ -342,15 +353,7 @@ vmod_saintmode__init(VRT_CTX, struct vmod_saintmode_saintmode **smp,
 	sm->be = be;
 	VTAILQ_INIT(&sm->troublelist);
 
-	sm->sdir->magic = DIRECTOR_MAGIC;
-	sm->sdir->resolve = resolve;
-	sm->sdir->healthy = healthy;
-#ifdef HAVE_DIRECTOR_ADMIN_HEALTH
-	sm->sdir->admin_health = VDI_AH_HEALTHY;
-#endif
-	REPLACE(sm->sdir->vcl_name, vcl_name);
-	sm->sdir->name = "saintmode";
-	sm->sdir->priv = sm;
+	sm->sdir = VRT_AddDirector(ctx, vmod_saintmode_methods, sm, "%s", vcl_name);
 
 	if (!priv->priv) {
 		ALLOC_OBJ(sm_objs, SAINTMODE_OBJS_MAGIC);
@@ -380,7 +383,7 @@ vmod_saintmode__fini(struct vmod_saintmode_saintmode **smp) {
 		FREE_OBJ(tr);
 	}
 
-	free(sm->sdir->vcl_name);
+	VRT_DelDirector(&sm->sdir);
 	AZ(pthread_mutex_destroy(&sm->mtx));
 
 	/* We can no longer refer to the sm_objs after this
